@@ -30,12 +30,12 @@ class AsgiTests(unittest.TestCase):
             'eio',
             'other_app',
             static_files='static_files',
-            engineio_path='/foo',
+            engineio_path='/foo/',
         )
         assert app.engineio_server == 'eio'
         assert app.other_asgi_app == 'other_app'
         assert app.static_files == 'static_files'
-        assert app.engineio_path == 'foo'
+        assert app.engineio_path == '/foo/'
 
     def test_engineio_routing(self):
         mock_server = mock.MagicMock()
@@ -54,6 +54,13 @@ class AsgiTests(unittest.TestCase):
         _run(app(scope, 'receive', 'send'))
         other_app.mock.assert_called_once_with(scope, 'receive', 'send')
 
+    def test_other_app_lifespan_routing(self):
+        other_app = AsyncMock()
+        app = async_asgi.ASGIApp('eio', other_app)
+        scope = {'type': 'lifespan'}
+        _run(app(scope, 'receive', 'send'))
+        other_app.mock.assert_called_once_with(scope, 'receive', 'send')
+
     def test_static_file_routing(self):
         root_dir = os.path.dirname(__file__)
         app = async_asgi.ASGIApp(
@@ -66,6 +73,8 @@ class AsgiTests(unittest.TestCase):
                 },
                 '/static': root_dir,
                 '/static/test/': root_dir + '/',
+                '/static2/test/': {'filename': root_dir + '/',
+                                   'content_type': 'image/gif'},
             },
         )
 
@@ -89,11 +98,29 @@ class AsgiTests(unittest.TestCase):
 
         check_path('/', 200, 'text/html', '<html></html>\n')
         check_path('/foo', 200, 'text/plain', '<html></html>\n')
+        check_path('/foo/bar', 404, 'text/plain', 'Not Found')
         check_path('/static/index.html', 200, 'text/html', '<html></html>\n')
         check_path('/static/foo.bar', 404, 'text/plain', 'Not Found')
         check_path(
             '/static/test/index.html', 200, 'text/html', '<html></html>\n'
         )
+        check_path('/static/test/index.html', 200, 'text/html',
+                   '<html></html>\n')
+        check_path('/static/test/files/', 200, 'text/html',
+                   '<html>file</html>\n')
+        check_path('/static/test/files/file.txt', 200, 'text/plain',
+                   'file\n')
+        check_path('/static/test/files/x.html', 404, 'text/plain',
+                   'Not Found')
+        check_path('/static2/test/', 200, 'image/gif', '<html></html>\n')
+        check_path('/static2/test/index.html', 200, 'image/gif',
+                   '<html></html>\n')
+        check_path('/static2/test/files/', 200, 'image/gif',
+                   '<html>file</html>\n')
+        check_path('/static2/test/files/file.txt', 200, 'image/gif',
+                   'file\n')
+        check_path('/static2/test/files/x.html', 404, 'text/plain',
+                   'Not Found')
         check_path('/bar/foo', 404, 'text/plain', 'Not Found')
         check_path('', 404, 'text/plain', 'Not Found')
 
@@ -431,7 +458,7 @@ class AsgiTests(unittest.TestCase):
         assert environ == {}
 
     def test_make_response(self):
-        environ = {'asgi.send': AsyncMock()}
+        environ = {'asgi.send': AsyncMock(), 'asgi.scope': {'type': 'http'}}
         _run(
             async_asgi.make_response(
                 '202 ACCEPTED', [('foo', 'bar')], b'payload', environ
@@ -451,7 +478,7 @@ class AsgiTests(unittest.TestCase):
     def test_make_response_websocket_accept(self):
         environ = {
             'asgi.send': AsyncMock(),
-            'HTTP_SEC_WEBSOCKET_VERSION': 'foo',
+            'asgi.scope': {'type': 'websocket'},
         }
         _run(
             async_asgi.make_response(
@@ -465,13 +492,27 @@ class AsgiTests(unittest.TestCase):
     def test_make_response_websocket_reject(self):
         environ = {
             'asgi.send': AsyncMock(),
-            'HTTP_SEC_WEBSOCKET_VERSION': 'foo',
+            'asgi.scope': {'type': 'websocket'},
         }
         _run(
             async_asgi.make_response(
                 '401 UNAUTHORIZED', [('foo', 'bar')], b'payload', environ
             )
         )
+        environ['asgi.send'].mock.assert_called_with(
+            {'type': 'websocket.close', 'reason': 'payload'}
+        )
+
+    def test_make_response_websocket_reject_no_payload(self):
+        environ = {
+            'asgi.send': AsyncMock(),
+            'asgi.scope': {'type': 'websocket'},
+        }
+        _run(
+            async_asgi.make_response(
+                '401 UNAUTHORIZED', [('foo', 'bar')], None, environ
+            )
+        )
         environ['asgi.send'].mock.assert_called_with(
             {'type': 'websocket.close'}
         )
