@@ -539,6 +539,31 @@ class TestAsyncClient(unittest.TestCase):
         c.http.ws_connect.mock.assert_called_once_with(
             'ws://foo/engine.io/?transport=websocket&EIO=4&t=123.456',
             headers={'Foo': 'Bar'},
+            timeout=5
+        )
+
+    @mock.patch('engineio.client.time.time', return_value=123.456)
+    def test_websocket_connection_extra(self, _time):
+        c = asyncio_client.AsyncClient(websocket_extra_options={
+            'headers': {'Baz': 'Qux'},
+            'timeout': 10
+        })
+        c.http = mock.MagicMock(closed=False)
+        c.http.ws_connect = AsyncMock(
+            side_effect=[aiohttp.client_exceptions.ServerConnectionError()]
+        )
+        with pytest.raises(exceptions.ConnectionError):
+            _run(
+                c.connect(
+                    'http://foo',
+                    transports=['websocket'],
+                    headers={'Foo': 'Bar'},
+                )
+            )
+        c.http.ws_connect.mock.assert_called_once_with(
+            'ws://foo/engine.io/?transport=websocket&EIO=4&t=123.456',
+            headers={'Foo': 'Bar', 'Baz': 'Qux'},
+            timeout=10,
         )
 
     @mock.patch('engineio.client.time.time', return_value=123.456)
@@ -553,6 +578,7 @@ class TestAsyncClient(unittest.TestCase):
         c.http.ws_connect.mock.assert_called_once_with(
             'ws://foo/engine.io/?transport=websocket&EIO=4&sid=123&t=123.456',
             headers={},
+            timeout=5,
         )
 
     def test_websocket_connection_no_open_packet(self):
@@ -605,6 +631,7 @@ class TestAsyncClient(unittest.TestCase):
         c.http.ws_connect.mock.assert_called_once_with(
             'ws://foo/engine.io/?transport=websocket&EIO=4&t=123.456',
             headers={},
+            timeout=5,
         )
 
     @mock.patch('engineio.client.time.time', return_value=123.456)
@@ -677,6 +704,7 @@ class TestAsyncClient(unittest.TestCase):
         c.http.ws_connect.mock.assert_called_once_with(
             'ws://foo/engine.io/?transport=websocket&EIO=4&t=123.456',
             headers={},
+            timeout=5,
         )
 
     @mock.patch('engineio.client.time.time', return_value=123.456)
@@ -711,6 +739,7 @@ class TestAsyncClient(unittest.TestCase):
         c.http.ws_connect.mock.assert_called_once_with(
             'ws://foo/engine.io/?transport=websocket&EIO=4&t=123.456',
             headers={},
+            timeout=5,
         )
         c.http.cookie_jar.update_cookies.assert_called_once_with(
             {'key': 'value', 'key2': 'value2', 'key3': '"value3="'}
@@ -752,6 +781,7 @@ class TestAsyncClient(unittest.TestCase):
         c.http.ws_connect.mock.assert_called_once_with(
             'ws://foo/engine.io/?transport=websocket&EIO=4&t=123.456',
             headers={'Foo': 'Bar'},
+            timeout=5,
         )
         c.http.cookie_jar.update_cookies.assert_called_once_with(
             {'key3': 'value3'}
@@ -1211,7 +1241,7 @@ class TestAsyncClient(unittest.TestCase):
             'POST',
             'http://foo',
             body=p.encode(),
-            headers={'Content-Type': 'application/octet-stream'},
+            headers={'Content-Type': 'text/plain'},
             timeout=5,
         )
 
@@ -1252,7 +1282,7 @@ class TestAsyncClient(unittest.TestCase):
             'POST',
             'http://foo',
             body=p.encode(),
-            headers={'Content-Type': 'application/octet-stream'},
+            headers={'Content-Type': 'text/plain'},
             timeout=5,
         )
 
@@ -1288,7 +1318,7 @@ class TestAsyncClient(unittest.TestCase):
             'POST',
             'http://foo',
             body=p.encode(),
-            headers={'Content-Type': 'application/octet-stream'},
+            headers={'Content-Type': 'text/plain'},
             timeout=5,
         )
         assert c.state == 'connected'
@@ -1316,7 +1346,7 @@ class TestAsyncClient(unittest.TestCase):
             'POST',
             'http://foo',
             body=p.encode(),
-            headers={'Content-Type': 'application/octet-stream'},
+            headers={'Content-Type': 'text/plain'},
             timeout=5,
         )
         assert c.state == 'connected'
@@ -1345,7 +1375,7 @@ class TestAsyncClient(unittest.TestCase):
             'POST',
             'http://foo',
             body=p.encode(),
-            headers={'Content-Type': 'application/octet-stream'},
+            headers={'Content-Type': 'text/plain'},
             timeout=5,
         )
         assert c.state == 'disconnected'
