@@ -1,7 +1,6 @@
 import asyncio
 import gzip
 import io
-import json
 import logging
 import sys
 import unittest
@@ -13,6 +12,7 @@ import pytest
 from engineio import asyncio_server
 from engineio.async_drivers import aiohttp as async_aiohttp
 from engineio import exceptions
+from engineio import json
 from engineio import packet
 from engineio import payload
 
@@ -278,9 +278,9 @@ class TestAsyncServer(unittest.TestCase):
         assert packets[0].data['pingTimeout'] == 123000
         assert packets[0].data['pingInterval'] == 456000
 
-    @mock.patch('engineio.asyncio_socket.AsyncSocket')
     @mock.patch('importlib.import_module')
-    def test_connect_bad_poll(self, import_module, AsyncSocket):
+    @mock.patch('engineio.asyncio_server.asyncio_socket.AsyncSocket')
+    def test_connect_bad_poll(self, AsyncSocket, import_module):
         a = self.get_async_mock()
         import_module.side_effect = [a]
         AsyncSocket.return_value = self._get_mock_socket()
@@ -290,9 +290,9 @@ class TestAsyncServer(unittest.TestCase):
         assert a._async['make_response'].call_count == 1
         assert a._async['make_response'].call_args[0][0] == '400 BAD REQUEST'
 
-    @mock.patch('engineio.asyncio_socket.AsyncSocket')
     @mock.patch('importlib.import_module')
-    def test_connect_transport_websocket(self, import_module, AsyncSocket):
+    @mock.patch('engineio.asyncio_server.asyncio_socket.AsyncSocket')
+    def test_connect_transport_websocket(self, AsyncSocket, import_module):
         a = self.get_async_mock(
             {
                 'REQUEST_METHOD': 'GET',
@@ -312,9 +312,9 @@ class TestAsyncServer(unittest.TestCase):
             == packet.OPEN
         )
 
-    @mock.patch('engineio.asyncio_socket.AsyncSocket')
     @mock.patch('importlib.import_module')
-    def test_http_upgrade_case_insensitive(self, import_module, AsyncSocket):
+    @mock.patch('engineio.asyncio_server.asyncio_socket.AsyncSocket')
+    def test_http_upgrade_case_insensitive(self, AsyncSocket, import_module):
         a = self.get_async_mock(
             {
                 'REQUEST_METHOD': 'GET',
@@ -334,11 +334,10 @@ class TestAsyncServer(unittest.TestCase):
             == packet.OPEN
         )
 
-    @mock.patch('engineio.asyncio_socket.AsyncSocket')
     @mock.patch('importlib.import_module')
+    @mock.patch('engineio.asyncio_server.asyncio_socket.AsyncSocket')
     def test_connect_transport_websocket_closed(
-        self, import_module, AsyncSocket
-    ):
+            self, AsyncSocket, import_module):
         a = self.get_async_mock(
             {
                 'REQUEST_METHOD': 'GET',
@@ -404,6 +403,29 @@ class TestAsyncServer(unittest.TestCase):
         headers = a._async['make_response'].call_args[0][1]
         assert ('Access-Control-Allow-Origin', 'b') in headers
 
+    @mock.patch('importlib.import_module')
+    def test_connect_cors_allowed_origin_with_callable(self, import_module):
+        def cors(origin):
+            return origin == 'a'
+
+        environ = {
+            'REQUEST_METHOD': 'GET',
+            'QUERY_STRING': '',
+            'HTTP_ORIGIN': 'a'
+        }
+        a = self.get_async_mock(environ)
+        import_module.side_effect = [a]
+
+        s = asyncio_server.AsyncServer(cors_allowed_origins=cors)
+        _run(s.handle_request('request'))
+        assert a._async['make_response'].call_args[0][0] == '200 OK'
+        headers = a._async['make_response'].call_args[0][1]
+        assert ('Access-Control-Allow-Origin', 'a') in headers
+
+        environ['HTTP_ORIGIN'] = 'b'
+        _run(s.handle_request('request'))
+        assert a._async['make_response'].call_args[0][0] == '400 BAD REQUEST'
+
     @mock.patch('importlib.import_module')
     def test_connect_cors_not_allowed_origin(self, import_module):
         a = self.get_async_mock(
@@ -1121,3 +1143,16 @@ class TestAsyncServer(unittest.TestCase):
         s._service_task = AsyncMock()
         _run(s.handle_request('request'))
         s._service_task.mock.assert_called_once_with()
+
+    @mock.patch('importlib.import_module')
+    def test_transports_disallowed(self, import_module):
+        a = self.get_async_mock(
+            {'REQUEST_METHOD': 'GET', 'QUERY_STRING': 'transport=polling'}
+        )
+        import_module.side_effect = [a]
+        s = asyncio_server.AsyncServer(transports='websocket')
+        response = _run(s.handle_request('request'))
+        assert response == 'response'
+        a._async['translate_request'].assert_called_once_with('request')
+        assert a._async['make_response'].call_count == 1
+        assert a._async['make_response'].call_args[0][0] == '400 BAD REQUEST'
