@@ -1,4 +1,3 @@
-import json
 import logging
 import ssl
 import time
@@ -10,6 +9,7 @@ import websocket
 
 from engineio import client
 from engineio import exceptions
+from engineio import json
 from engineio import packet
 from engineio import payload
 
@@ -59,7 +59,7 @@ class TestClient(unittest.TestCase):
         c = client.Client(logger=my_logger)
         assert c.logger == my_logger
 
-    def test_custon_timeout(self):
+    def test_custom_timeout(self):
         c = client.Client()
         assert c.request_timeout == 5
         c = client.Client(request_timeout=27)
@@ -558,6 +558,27 @@ class TestClient(unittest.TestCase):
             header={'Foo': 'Bar'},
             cookie=None,
             enable_multithread=True,
+            timeout=5
+        )
+
+    @mock.patch('engineio.client.time.time', return_value=123.456)
+    @mock.patch(
+        'engineio.client.websocket.create_connection',
+        side_effect=[ConnectionError],
+    )
+    def test_websocket_connection_extra(self, create_connection, _time):
+        c = client.Client(websocket_extra_options={'header': {'Baz': 'Qux'},
+                                                   'timeout': 10})
+        with pytest.raises(exceptions.ConnectionError):
+            c.connect(
+                'http://foo', transports=['websocket'], headers={'Foo': 'Bar'}
+            )
+        create_connection.assert_called_once_with(
+            'ws://foo/engine.io/?transport=websocket&EIO=4&t=123.456',
+            header={'Foo': 'Bar', 'Baz': 'Qux'},
+            cookie=None,
+            enable_multithread=True,
+            timeout=10
         )
 
     @mock.patch('engineio.client.time.time', return_value=123.456)
@@ -578,6 +599,7 @@ class TestClient(unittest.TestCase):
             header={'Foo': 'Bar'},
             cookie=None,
             enable_multithread=True,
+            timeout=5
         )
 
     @mock.patch('engineio.client.time.time', return_value=123.456)
@@ -596,6 +618,7 @@ class TestClient(unittest.TestCase):
             header={},
             cookie=None,
             enable_multithread=True,
+            timeout=5
         )
 
     @mock.patch('engineio.client.websocket.create_connection')
@@ -644,6 +667,7 @@ class TestClient(unittest.TestCase):
             'header': {},
             'cookie': None,
             'enable_multithread': True,
+            'timeout': 5,
         }
 
     @mock.patch('engineio.client.websocket.create_connection')
@@ -685,6 +709,7 @@ class TestClient(unittest.TestCase):
             'header': {},
             'cookie': None,
             'enable_multithread': True,
+            'timeout': 5,
             'sslopt': {'cert_reqs': ssl.CERT_NONE},
         }
 
@@ -722,6 +747,7 @@ class TestClient(unittest.TestCase):
             'header': {},
             'cookie': 'key=value; key2=value2',
             'enable_multithread': True,
+            'timeout': 5,
         }
 
     @mock.patch('engineio.client.websocket.create_connection')
@@ -758,6 +784,7 @@ class TestClient(unittest.TestCase):
             'header': {'Foo': 'bar'},
             'cookie': 'key=value',
             'enable_multithread': True,
+            'timeout': 5,
         }
 
     @mock.patch('engineio.client.websocket.create_connection')
@@ -799,6 +826,7 @@ class TestClient(unittest.TestCase):
         assert create_connection.call_args[1] == {
             'header': {},
             'enable_multithread': True,
+            'timeout': 5,
             'cookie': 'key=value; key2=value2; key3=value3',
         }
 
@@ -831,6 +859,7 @@ class TestClient(unittest.TestCase):
             'header': {'Authorization': 'Basic Zm9vOmJhcg=='},
             'cookie': '',
             'enable_multithread': True,
+            'timeout': 5,
         }
 
     @mock.patch('engineio.client.websocket.create_connection')
@@ -863,6 +892,7 @@ class TestClient(unittest.TestCase):
             'header': {},
             'cookie': '',
             'enable_multithread': True,
+            'timeout': 5,
         }
 
     @mock.patch('engineio.client.websocket.create_connection')
@@ -895,6 +925,47 @@ class TestClient(unittest.TestCase):
             'header': {},
             'cookie': '',
             'enable_multithread': True,
+            'timeout': 5,
+        }
+
+    @mock.patch('engineio.client.websocket.create_connection')
+    def test_websocket_connection_verify_with_cert_and_key(
+        self, create_connection
+    ):
+        create_connection.return_value.recv.return_value = packet.Packet(
+            packet.OPEN,
+            {
+                'sid': '123',
+                'upgrades': [],
+                'pingInterval': 1000,
+                'pingTimeout': 2000,
+            },
+        ).encode()
+        http = mock.MagicMock()
+        http.cookies = []
+        http.auth = None
+        http.proxies = None
+        http.cert = ('foo.crt', 'key.pem')
+        http.verify = 'ca-bundle.crt'
+        c = client.Client(http_session=http)
+        c._read_loop_polling = mock.MagicMock()
+        c._read_loop_websocket = mock.MagicMock()
+        c._write_loop = mock.MagicMock()
+        on_connect = mock.MagicMock()
+        c.on('connect', on_connect)
+        c.connect('ws://foo', transports=['websocket'])
+
+        assert len(create_connection.call_args_list) == 1
+        assert create_connection.call_args[1] == {
+            'sslopt': {
+                'certfile': 'foo.crt',
+                'keyfile': 'key.pem',
+                'ca_certs': 'ca-bundle.crt'
+            },
+            'header': {},
+            'cookie': '',
+            'enable_multithread': True,
+            'timeout': 5,
         }
 
     @mock.patch('engineio.client.websocket.create_connection')
@@ -955,6 +1026,7 @@ class TestClient(unittest.TestCase):
                 'header': {},
                 'cookie': '',
                 'enable_multithread': True,
+                'timeout': 5,
             }
             if results:
                 expected_results.update({
@@ -994,6 +1066,41 @@ class TestClient(unittest.TestCase):
             'header': {},
             'cookie': '',
             'enable_multithread': True,
+            'timeout': 5,
+        }
+
+    @mock.patch('engineio.client.websocket.create_connection')
+    def test_websocket_connection_with_verify(self, create_connection):
+        create_connection.return_value.recv.return_value = packet.Packet(
+            packet.OPEN,
+            {
+                'sid': '123',
+                'upgrades': [],
+                'pingInterval': 1000,
+                'pingTimeout': 2000,
+            },
+        ).encode()
+        http = mock.MagicMock()
+        http.cookies = []
+        http.auth = None
+        http.proxies = None
+        http.cert = None
+        http.verify = 'ca-bundle.crt'
+        c = client.Client(http_session=http)
+        c._read_loop_polling = mock.MagicMock()
+        c._read_loop_websocket = mock.MagicMock()
+        c._write_loop = mock.MagicMock()
+        on_connect = mock.MagicMock()
+        c.on('connect', on_connect)
+        c.connect('ws://foo', transports=['websocket'])
+
+        assert len(create_connection.call_args_list) == 1
+        assert create_connection.call_args[1] == {
+            'sslopt': {'ca_certs': 'ca-bundle.crt'},
+            'header': {},
+            'cookie': '',
+            'enable_multithread': True,
+            'timeout': 5,
         }
 
     @mock.patch('engineio.client.websocket.create_connection')
@@ -1394,7 +1501,7 @@ class TestClient(unittest.TestCase):
             'POST',
             'http://foo',
             body=p.encode(),
-            headers={'Content-Type': 'application/octet-stream'},
+            headers={'Content-Type': 'text/plain'},
             timeout=5,
         )
 
@@ -1429,7 +1536,7 @@ class TestClient(unittest.TestCase):
             'POST',
             'http://foo',
             body=p.encode(),
-            headers={'Content-Type': 'application/octet-stream'},
+            headers={'Content-Type': 'text/plain'},
             timeout=5,
         )
 
@@ -1462,7 +1569,7 @@ class TestClient(unittest.TestCase):
             'POST',
             'http://foo',
             body=p.encode(),
-            headers={'Content-Type': 'application/octet-stream'},
+            headers={'Content-Type': 'text/plain'},
             timeout=5,
         )
         assert c.state == 'connected'
@@ -1491,7 +1598,7 @@ class TestClient(unittest.TestCase):
             'POST',
             'http://foo',
             body=p.encode(),
-            headers={'Content-Type': 'application/octet-stream'},
+            headers={'Content-Type': 'text/plain'},
             timeout=5,
         )
         assert c.state == 'connected'
@@ -1520,7 +1627,7 @@ class TestClient(unittest.TestCase):
             'POST',
             'http://foo',
             body=p.encode(),
-            headers={'Content-Type': 'application/octet-stream'},
+            headers={'Content-Type': 'text/plain'},
             timeout=5,
         )
         assert c.state == 'disconnected'
