@@ -1,7 +1,6 @@
 import gzip
 import importlib
 import io
-import json
 import logging
 import sys
 import time
@@ -12,6 +11,7 @@ import zlib
 import pytest
 
 from engineio import exceptions
+from engineio import json
 from engineio import packet
 from engineio import payload
 from engineio import server
@@ -97,19 +97,31 @@ class TestServer(unittest.TestCase):
         server.Server(foo='bar')  # this should not raise
 
     def test_async_mode_threading(self):
+        sys.modules['simple_websocket'] = mock.MagicMock()
         s = server.Server(async_mode='threading')
         assert s.async_mode == 'threading'
 
         import threading
+        from engineio.async_drivers import threading as async_threading
+        import queue
 
-        try:
-            import queue
-        except ImportError:
-            import Queue as queue
+        assert s._async['thread'] == threading.Thread
+        assert s._async['queue'] == queue.Queue
+        assert s._async['websocket'] == async_threading.WebSocketWSGI
+        del sys.modules['simple_websocket']
+        del sys.modules['engineio.async_drivers.threading']
+
+    def test_async_mode_threading_without_websocket(self):
+        s = server.Server(async_mode='threading')
+        assert s.async_mode == 'threading'
+
+        import threading
+        import queue
 
         assert s._async['thread'] == threading.Thread
         assert s._async['queue'] == queue.Queue
         assert s._async['websocket'] is None
+        del sys.modules['engineio.async_drivers.threading']
 
     def test_async_mode_eventlet(self):
         s = server.Server(async_mode='eventlet')
@@ -574,6 +586,26 @@ class TestServer(unittest.TestCase):
         headers = start_response.call_args[0][1]
         assert ('Access-Control-Allow-Origin', 'b') in headers
 
+    def test_connect_cors_allowed_origin_with_callable(self):
+        def cors(origin):
+            return origin == 'a'
+
+        s = server.Server(cors_allowed_origins=cors)
+        environ = {
+            'REQUEST_METHOD': 'GET',
+            'QUERY_STRING': 'EIO=4',
+            'HTTP_ORIGIN': 'a',
+        }
+        start_response = mock.MagicMock()
+        s.handle_request(environ, start_response)
+        assert start_response.call_args[0][0] == '200 OK'
+        headers = start_response.call_args[0][1]
+        assert ('Access-Control-Allow-Origin', 'a') in headers
+
+        environ['HTTP_ORIGIN'] = 'b'
+        s.handle_request(environ, start_response)
+        assert start_response.call_args[0][0] == '400 BAD REQUEST'
+
     def test_connect_cors_not_allowed_origin(self):
         s = server.Server(cors_allowed_origins=['a', 'b'])
         environ = {
@@ -646,6 +678,22 @@ class TestServer(unittest.TestCase):
         assert ('Access-Control-Allow-Origin', 'http://foo') in headers
 
     def test_connect_cors_headers_default_origin_proxy_server(self):
+        s = server.Server()
+        environ = {
+            'REQUEST_METHOD': 'GET',
+            'QUERY_STRING': 'EIO=4',
+            'wsgi.url_scheme': 'http',
+            'HTTP_HOST': 'foo',
+            'HTTP_ORIGIN': 'https://foo',
+            'HTTP_X_FORWARDED_PROTO': 'https, ftp',
+        }
+        start_response = mock.MagicMock()
+        s.handle_request(environ, start_response)
+        assert start_response.call_args[0][0] == '200 OK'
+        headers = start_response.call_args[0][1]
+        assert ('Access-Control-Allow-Origin', 'https://foo') in headers
+
+    def test_connect_cors_headers_default_origin_proxy_server2(self):
         s = server.Server()
         environ = {
             'REQUEST_METHOD': 'GET',
@@ -1121,3 +1169,19 @@ class TestServer(unittest.TestCase):
         start_response = mock.MagicMock()
         s.handle_request(environ, start_response)
         s._service_task.assert_called_once_with()
+
+    def test_transports_invalid(self):
+        with pytest.raises(ValueError):
+            server.Server(transports='invalid')
+        with pytest.raises(ValueError):
+            server.Server(transports=['invalid', 'foo'])
+
+    def test_transports_disallowed(self):
+        s = server.Server(transports='websocket')
+        environ = {
+            'REQUEST_METHOD': 'GET',
+            'QUERY_STRING': 'transport=polling',
+        }
+        start_response = mock.MagicMock()
+        s.handle_request(environ, start_response)
+        assert start_response.call_args[0][0] == '400 BAD REQUEST'
